package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TagConsumeRecordBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeAmountRequest;
import com.gic.spark.util.CommonUtil;
import com.gic.spark.util.DateUtil;
import org.apache.spark.api.java.JavaRDD;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/6
 */
public class TagLatelyConsumptionMoneyFilter  extends AbstractTagConsumRecordFilter{

    private static TagLatelyConsumptionMoneyFilter instance;

    public static TagLatelyConsumptionMoneyFilter getInstance() {
        if(null==instance){
            instance=new TagLatelyConsumptionMoneyFilter();
        }
        return instance;
    }
    private TagLatelyConsumptionMoneyFilter(){}

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHive);
        return result;
    }

    @Override
    public JavaRDD<Integer> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeAmountRequest consumeAmountRequest=(TagConsumeAmountRequest)request;
        JavaRDD<TagConsumeRecordBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHive.getDatasetByEnterpriseId(enterpriseId), TagConsumeRecordBean.class).javaRDD();
        consumeRecordRDD=statisticsTypeHandle(consumeRecordRDD,consumeAmountRequest);
        int configStatus= CommonUtil.getConfigStatus(enterpriseId);

        JavaRDD<Integer>ecuRdd=consumeRecordRDD.mapToPair(data-> Tuple2.apply(data.getEcu_id(),data))
                .groupByKey()
                .mapPartitions(data->{
                    List<Integer>result=new ArrayList();
                    while (data.hasNext()){

                        Tuple2<Integer, Iterable<TagConsumeRecordBean>> tp2=data.next();
                        double latelyConsumAmount=0;
                        TagConsumeRecordBean latelyConsumeAmountBean=null;
                        Map<String,TagConsumeRecordBean> effectiveOrderMap=new HashMap();
                        List<TagConsumeRecordBean>noEffectiveOrderList=new ArrayList();
                        tp2._2().forEach(element->{
                            if(element.getPay_amt()>0){
                                effectiveOrderMap.put(element.getOorder_no(),element);
                            }else{
                                noEffectiveOrderList.add(element);
                            }
                        });
                        noEffectiveOrderList.forEach(noEffectiveOrder->{
                            TagConsumeRecordBean effectiveOrder=effectiveOrderMap.get(noEffectiveOrder.getOorder_no());
                            if(noEffectiveOrder.getReceipts_time().substring(0,10).equals(effectiveOrder.getReceipts_time().substring(0,10))){
                                effectiveOrder.setPaid_amt(effectiveOrder.getPaid_amt()+noEffectiveOrder.getPaid_amt());
                                effectiveOrder.setPay_amt(effectiveOrder.getPay_amt()+noEffectiveOrder.getPay_amt());
                            }
                        });
                        for(TagConsumeRecordBean amountBean:effectiveOrderMap.values()){
                            if(null==latelyConsumeAmountBean){
                                latelyConsumeAmountBean=amountBean;
                            }else{
                                if(DateUtil.stringToDate(amountBean.getReceipts_time()).getTime()
                                        >DateUtil.stringToDate(latelyConsumeAmountBean.getReceipts_time()).getTime()){
                                    latelyConsumeAmountBean=amountBean;
                                }
                            }
                        }
                        latelyConsumAmount=null!=latelyConsumeAmountBean?
                                (configStatus==1?latelyConsumeAmountBean.getPaid_amt():latelyConsumeAmountBean.getPay_amt())
                                :latelyConsumAmount;

                        switch (consumeAmountRequest.getNumberType()){
                            case between:
                                if(latelyConsumAmount>=consumeAmountRequest.getBeginNum()
                                        &&latelyConsumAmount<=consumeAmountRequest.getEndNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            case lt:
                                if(latelyConsumAmount<consumeAmountRequest.getEndNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            case gt:
                                if(latelyConsumAmount>consumeAmountRequest.getBeginNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            case eq:
                                if(latelyConsumAmount==consumeAmountRequest.getEqualNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            case lte:
                                if(latelyConsumAmount<=consumeAmountRequest.getEndNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            case gte:
                                if(latelyConsumAmount>=consumeAmountRequest.getBeginNum()){
                                    result.add(tp2._1());
                                }
                                break;
                            default:break;
                        }
                    }
                    return result.iterator();
                });
        return ecuRdd;
    }
}
